import { NgZone } from '@angular/core';
import { Observable } from 'rxjs';
import { database } from 'firebase';
import { FirebaseDatabase } from './firebase.service';
import { DataSnapshot } from './reexports';
import './add/run-in-zone';
export declare type DatabaseReference = database.Reference;
export declare type Query = database.Query;
export declare type EventType = 'value' | 'child_added' | 'child_changed' | 'child_removed' | 'child_moved';
export declare class Event {
    static Value: EventType;
    static ChildAdded: EventType;
    static ChildChanged: EventType;
    static ChildRemoved: EventType;
    static ChildMoved: EventType;
}
export interface ExtendedDataSnapshot extends DataSnapshot {
    prevKey: string;
}
export declare class DataSnapshotObservable<T> extends Observable<ExtendedDataSnapshot> {
    exists(): Observable<boolean>;
    children<C>(): Observable<DataSnapshotObservable<C>>;
    /**
     * This operator takes the result of .val() for all children of the snapshot and emits
     * them as an array.
     * Contents of source snapshot:
     * ```
     * {
     *  childA: { prop: 'Hello' },
     *  childB: { prop: 'World!' },
     * }
     * ```
     * Result of operator:
     * ```
     * [
     *  { prop: 'Hello' },
     *  { prop: 'World!' },
     * ]
     * ```
     * @returns {Observable<C[]>}
     */
    toValArray<C>(): Observable<C[]>;
    key(): Observable<string>;
    prevKey(): Observable<string>;
    val<T>(): Observable<T>;
    getPriority(): Observable<number | string>;
    exportVal(): Observable<any>;
    hasChild(path: string): Observable<boolean>;
    hasChildren(): Observable<boolean>;
    numChildren(): Observable<number>;
    child<C>(path: string): DataSnapshotObservable<C>;
}
export declare class FirebaseQuery {
    protected _ref: DatabaseReference;
    protected ngZone: NgZone;
    private query;
    protected wrappedRef: FirebaseDatabaseRef;
    readonly ref: FirebaseDatabaseRef;
    constructor(_ref: DatabaseReference, ngZone: NgZone);
    orderByChild(child: string): FirebaseQuery;
    orderByKey(): FirebaseQuery;
    orderByPriority(): FirebaseQuery;
    orderByValue(): FirebaseQuery;
    startAt(value: number | string | boolean | null, key?: string): FirebaseQuery;
    endAt(value: number | string | boolean | null, key?: string): FirebaseQuery;
    equalTo(value: number | string | boolean | null, key?: string): FirebaseQuery;
    limitToFirst(limit: number): FirebaseQuery;
    limitToLast(limit: number): FirebaseQuery;
    once<T>(event: EventType): DataSnapshotObservable<T>;
    onceValue<T>(): DataSnapshotObservable<T>;
    onceChildAdded<T>(): DataSnapshotObservable<T>;
    onceChildChanged<T>(): DataSnapshotObservable<T>;
    onceChildMoved<T>(): DataSnapshotObservable<T>;
    onceChildRemoved<T>(): DataSnapshotObservable<T>;
    on<T>(event: EventType): DataSnapshotObservable<T>;
    onValue<T>(): DataSnapshotObservable<T>;
    onChildAdded<T>(): DataSnapshotObservable<T>;
    onChildChanged<T>(): DataSnapshotObservable<T>;
    onChildMoved<T>(): DataSnapshotObservable<T>;
    onChildRemoved<T>(): DataSnapshotObservable<T>;
    isEqual(query: FirebaseQuery): boolean;
    private getEventHandler(sub, complete?);
    private getQueryOrRef();
    private _call(fnName, ...args);
}
export declare class FirebaseDatabaseRef extends FirebaseQuery {
    protected ngZone: NgZone;
    parent: FirebaseDatabaseRef;
    root: FirebaseDatabaseRef;
    readonly key: string | null;
    constructor(_ref: DatabaseReference, ngZone: NgZone, parent: FirebaseDatabaseRef, root: FirebaseDatabaseRef);
    child(path: string): FirebaseDatabaseRef;
    set(value: any): Observable<void>;
    setPriority(priority: string | number | null): Observable<void>;
    setWithPriority(newVal: any, priority: string | number | null): Observable<void>;
    push(value?: any): Observable<FirebaseDatabaseRef>;
    update(value: Object): Observable<void>;
    remove(): Observable<void>;
    transaction(transactionUpdate: (node: any) => any | never, applyLocally?: boolean): Observable<{
        committed: boolean;
        snapshot: DataSnapshot;
    }>;
}
export declare class FirebaseDatabaseService {
    private fbDatabase;
    private ngZone;
    constructor(fbDatabase: FirebaseDatabase, ngZone: NgZone);
    ref(path?: string): FirebaseDatabaseRef;
}
