var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { Injectable, NgZone } from '@angular/core';
import { Observable } from 'rxjs';
import { FirebaseDatabase } from './firebase.service';
import { wrapPromise } from './utils';
import './add/run-in-zone';
export var Event = (function () {
    function Event() {
    }
    Event.Value = 'value';
    Event.ChildAdded = 'child_added';
    Event.ChildChanged = 'child_changed';
    Event.ChildRemoved = 'child_removed';
    Event.ChildMoved = 'child_moved';
    return Event;
}());
export var DataSnapshotObservable = (function (_super) {
    __extends(DataSnapshotObservable, _super);
    function DataSnapshotObservable() {
        _super.apply(this, arguments);
    }
    DataSnapshotObservable.prototype.exists = function () {
        return this.map(function (snapshot) { return snapshot.exists(); });
    };
    DataSnapshotObservable.prototype.children = function () {
        return this.map(function (snapshot) { return new DataSnapshotObservable(function (sub) {
            snapshot.forEach(function (childSnapshot) { return sub.next(childSnapshot); });
            sub.complete();
        }); });
    };
    /**
     * This operator takes the result of .val() for all children of the snapshot and emits
     * them as an array.
     * Contents of source snapshot:
     * ```
     * {
     *  childA: { prop: 'Hello' },
     *  childB: { prop: 'World!' },
     * }
     * ```
     * Result of operator:
     * ```
     * [
     *  { prop: 'Hello' },
     *  { prop: 'World!' },
     * ]
     * ```
     * @returns {Observable<C[]>}
     */
    DataSnapshotObservable.prototype.toValArray = function () {
        return this.children()
            .switchMap(function (children) { return children
            .val()
            .toArray(); });
    };
    DataSnapshotObservable.prototype.key = function () {
        return this.map(function (snapshot) { return snapshot.key; });
    };
    DataSnapshotObservable.prototype.prevKey = function () {
        return this.map(function (snapshot) { return snapshot.prevKey; });
    };
    DataSnapshotObservable.prototype.val = function () {
        return this.map(function (snapshot) { return snapshot.val(); });
    };
    DataSnapshotObservable.prototype.getPriority = function () {
        return this.map(function (snapshot) { return snapshot.getPriority(); });
    };
    DataSnapshotObservable.prototype.exportVal = function () {
        return this.map(function (snapshot) { return snapshot.exportVal(); });
    };
    DataSnapshotObservable.prototype.hasChild = function (path) {
        return this.map(function (snapshot) { return snapshot.hasChild(path); });
    };
    DataSnapshotObservable.prototype.hasChildren = function () {
        return this.map(function (snapshot) { return snapshot.hasChildren(); });
    };
    DataSnapshotObservable.prototype.numChildren = function () {
        return this.map(function (snapshot) { return snapshot.numChildren(); });
    };
    DataSnapshotObservable.prototype.child = function (path) {
        var _this = this;
        return new DataSnapshotObservable(function (sub) {
            var subscription = _this
                .map(function (snapshot) { return snapshot.child(path); })
                .subscribe(sub);
            return function () { return subscription.unsubscribe(); };
        });
    };
    return DataSnapshotObservable;
}(Observable));
export var FirebaseQuery = (function () {
    function FirebaseQuery(_ref, ngZone) {
        this._ref = _ref;
        this.ngZone = ngZone;
    }
    Object.defineProperty(FirebaseQuery.prototype, "ref", {
        get: function () {
            return this.wrappedRef;
        },
        enumerable: true,
        configurable: true
    });
    FirebaseQuery.prototype.orderByChild = function (child) {
        this._call('orderByChild', child);
        return this;
    };
    FirebaseQuery.prototype.orderByKey = function () {
        this._call('orderByKey');
        return this;
    };
    FirebaseQuery.prototype.orderByPriority = function () {
        this._call('orderByPriority');
        return this;
    };
    FirebaseQuery.prototype.orderByValue = function () {
        this._call('orderByValue');
        return this;
    };
    FirebaseQuery.prototype.startAt = function (value, key) {
        this._call('startAt', value, key);
        return this;
    };
    FirebaseQuery.prototype.endAt = function (value, key) {
        this._call('endAt', value, key);
        return this;
    };
    FirebaseQuery.prototype.equalTo = function (value, key) {
        this._call('equalTo', value, key);
        return this;
    };
    FirebaseQuery.prototype.limitToFirst = function (limit) {
        this._call('limitToFirst', limit);
        return this;
    };
    FirebaseQuery.prototype.limitToLast = function (limit) {
        this._call('limitToLast', limit);
        return this;
    };
    FirebaseQuery.prototype.once = function (event) {
        var _this = this;
        return new DataSnapshotObservable(function (sub) {
            _this.getQueryOrRef().once(event, _this.getEventHandler(sub, true), function (err) { return function () { return sub.error(err); }; });
        }).runInZone(this.ngZone);
    };
    FirebaseQuery.prototype.onceValue = function () {
        return this.once(Event.Value);
    };
    FirebaseQuery.prototype.onceChildAdded = function () {
        return this.once(Event.ChildAdded);
    };
    FirebaseQuery.prototype.onceChildChanged = function () {
        return this.once(Event.ChildChanged);
    };
    FirebaseQuery.prototype.onceChildMoved = function () {
        return this.once(Event.ChildMoved);
    };
    FirebaseQuery.prototype.onceChildRemoved = function () {
        return this.once(Event.ChildRemoved);
    };
    FirebaseQuery.prototype.on = function (event) {
        var _this = this;
        return new DataSnapshotObservable(function (sub) {
            var cb = _this.getQueryOrRef().on(event, _this.getEventHandler(sub), function (err) { return sub.error(err); });
            return function () { return _this.getQueryOrRef().off(event, cb); };
        }).runInZone(this.ngZone);
    };
    FirebaseQuery.prototype.onValue = function () {
        return this.on(Event.Value);
    };
    FirebaseQuery.prototype.onChildAdded = function () {
        return this.on(Event.ChildAdded);
    };
    FirebaseQuery.prototype.onChildChanged = function () {
        return this.on(Event.ChildChanged);
    };
    FirebaseQuery.prototype.onChildMoved = function () {
        return this.on(Event.ChildMoved);
    };
    FirebaseQuery.prototype.onChildRemoved = function () {
        return this.on(Event.ChildRemoved);
    };
    FirebaseQuery.prototype.isEqual = function (query) {
        return this.getQueryOrRef().isEqual(query.getQueryOrRef());
    };
    FirebaseQuery.prototype.getEventHandler = function (sub, complete) {
        return function (snapshot, prevKey) {
            snapshot.prevKey = prevKey;
            sub.next(snapshot);
            if (complete) {
                sub.complete();
            }
        };
    };
    FirebaseQuery.prototype.getQueryOrRef = function () {
        if (this.query) {
            return this.query;
        }
        return this._ref;
    };
    FirebaseQuery.prototype._call = function (fnName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (this.query) {
            this.query = (_a = this.query)[fnName].apply(_a, args);
        }
        else {
            this.query = (_b = this._ref)[fnName].apply(_b, args);
        }
        var _a, _b;
    };
    return FirebaseQuery;
}());
export var FirebaseDatabaseRef = (function (_super) {
    __extends(FirebaseDatabaseRef, _super);
    function FirebaseDatabaseRef(_ref, ngZone, parent, root) {
        _super.call(this, _ref, ngZone);
        this.ngZone = ngZone;
        this.parent = parent;
        this.root = root;
        this.root = root || this;
        this.wrappedRef = this;
    }
    Object.defineProperty(FirebaseDatabaseRef.prototype, "key", {
        get: function () {
            return this._ref.key;
        },
        enumerable: true,
        configurable: true
    });
    FirebaseDatabaseRef.prototype.child = function (path) {
        var childRef = this._ref.child(path);
        return new FirebaseDatabaseRef(childRef, this.ngZone, this, this.root);
    };
    FirebaseDatabaseRef.prototype.set = function (value) {
        var _this = this;
        return wrapPromise(function () { return _this._ref.set(value); });
    };
    FirebaseDatabaseRef.prototype.setPriority = function (priority) {
        var _this = this;
        // There seems to be a bug with the typing for #setPriority(priority, onComplete): Promise
        // The firebase library, in every other case, declares the onComplete function optional since a
        // Promise is returned as well.
        return wrapPromise(function () { return _this._ref.setPriority(priority); });
    };
    FirebaseDatabaseRef.prototype.setWithPriority = function (newVal, priority) {
        var _this = this;
        return wrapPromise(function () { return _this._ref.setWithPriority(newVal, priority); });
    };
    FirebaseDatabaseRef.prototype.push = function (value) {
        var pushRef = this._ref.push(value);
        var ref = new FirebaseDatabaseRef(pushRef, this.ngZone, this, this.root);
        // Only if a value to push was given, use ref as promise, since otherwise
        // pushRef.then will be undefined
        if (value) {
            return wrapPromise(function () { return pushRef; })
                .mapTo(ref);
        }
        return Observable.of(ref);
    };
    FirebaseDatabaseRef.prototype.update = function (value) {
        var _this = this;
        return wrapPromise(function () { return _this._ref.update(value); });
    };
    FirebaseDatabaseRef.prototype.remove = function () {
        var _this = this;
        return wrapPromise(function () { return _this._ref.remove(); });
    };
    FirebaseDatabaseRef.prototype.transaction = function (transactionUpdate, applyLocally) {
        var _this = this;
        return wrapPromise(function () { return new Promise(function (resolve, reject) { return _this._ref.transaction(transactionUpdate, function (err, committed, snapshot) { return err ? reject(err) : resolve({ committed: committed, snapshot: snapshot }); }, applyLocally); }); });
    };
    return FirebaseDatabaseRef;
}(FirebaseQuery));
export var FirebaseDatabaseService = (function () {
    function FirebaseDatabaseService(fbDatabase, ngZone) {
        this.fbDatabase = fbDatabase;
        this.ngZone = ngZone;
    }
    FirebaseDatabaseService.prototype.ref = function (path) {
        return new FirebaseDatabaseRef(this.fbDatabase.ref(path), this.ngZone, null, null);
    };
    FirebaseDatabaseService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    FirebaseDatabaseService.ctorParameters = [
        { type: FirebaseDatabase, },
        { type: NgZone, },
    ];
    return FirebaseDatabaseService;
}());
//# sourceMappingURL=firebase-database.service.js.map