import { NgModule, NgZone, OpaqueToken, Injectable, Inject, Optional } from '@angular/core';
import { LogService, LogServiceDummy } from '@blaugold/angular-logger';
import * as firebase from 'firebase';
import { FirebaseAuthService } from './firebase-auth.service';
import { FirebaseDatabaseService } from './firebase-database.service';
var FIREBASE_APP_CONFIGS = new OpaqueToken('FirebaseAppConfigs');
var lazyInvocation = true;
export function invokeLazy() {
    return lazyInvocation;
}
function setLazyInvocation(lazy) {
    lazyInvocation = lazy;
}
export var FirebaseModule = (function () {
    function FirebaseModule() {
    }
    FirebaseModule.forRoot = function (config, lazyInvocation) {
        if (lazyInvocation === void 0) { lazyInvocation = true; }
        setLazyInvocation(lazyInvocation);
        return {
            ngModule: FirebaseModule,
            providers: [
                { provide: FIREBASE_APP_CONFIGS, useValue: config },
                Firebase
            ]
        };
    };
    FirebaseModule.decorators = [
        { type: NgModule, args: [{},] },
    ];
    /** @nocollapse */
    FirebaseModule.ctorParameters = [];
    return FirebaseModule;
}());
export var Firebase = (function () {
    function Firebase(configs, ngZone, logService) {
        var _this = this;
        this.configs = configs;
        this.ngZone = ngZone;
        this.apps = {};
        this.instId = ++Firebase.lastInstanceId;
        this.defaultAppName = "default-" + this.instId;
        logService = logService || new LogServiceDummy();
        this.logger = logService.getLogger('Firebase');
        configs.forEach(function (config) {
            var name = config.name || _this.defaultAppName;
            _this.logger.info("Setting up project \"" + Firebase.getProjectName(config) + "\" as app \"" + name + "\"");
            if (_this.hasApp(name)) {
                throw new Error("Firebase: App with name: \"" + name + "\" already exists.");
            }
            var app = firebase.initializeApp(config.config, name);
            _this.apps[name] = {
                auth: new FirebaseAuthService(app.auth(), _this.ngZone),
                database: new FirebaseDatabaseService(app.database(), _this.ngZone)
            };
        });
    }
    Firebase.getProjectName = function (config) {
        return config.config.authDomain.split('.')[0];
    };
    Firebase.prototype.auth = function (appName) {
        if (appName === void 0) { appName = this.defaultAppName; }
        this.assertAppExists(appName);
        return this.apps[appName].auth;
    };
    Firebase.prototype.database = function (appName) {
        if (appName === void 0) { appName = this.defaultAppName; }
        this.assertAppExists(appName);
        return this.apps[appName].database;
    };
    Firebase.prototype.assertAppExists = function (appName) {
        if (!this.apps[appName]) {
            throw new Error("Firebase app: " + appName + " does not exist.'");
        }
    };
    Firebase.prototype.hasApp = function (appName) {
        return firebase.apps.some(function (app) { return app.name === appName; });
    };
    Firebase.lastInstanceId = -1;
    Firebase.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Firebase.ctorParameters = [
        { type: Array, decorators: [{ type: Inject, args: [FIREBASE_APP_CONFIGS,] },] },
        { type: NgZone, },
        { type: LogService, decorators: [{ type: Optional },] },
    ];
    return Firebase;
}());
//# sourceMappingURL=firebase.module.js.map